#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#define ESC_KEY 27

#define VIDEO_W 320					//画面の幅
#define VIDEO_H 240					//画面の高さ
#define VIDEO_BIT 24	
#define IMAGE_ENEMY image[0]		//敵の画像
#define IMAGE_BACK image[1]			//背景画像
#define IMAGE_AIM image[2]			//カーゾル画像
#define IMAGE_HIT image[3]			//弾痕画像
#define IMAGE_MARK_G image[4]		//頭にヒット時の画像
#define IMAGE_MARK_Y image[5]		//体にヒット時の画像
#define IMAGE_MARK_R image[6]		//足にヒット時の画像
#define IMAGE_AMMO image[7]			//残弾の画像
#define IMAGE_WAIT image[8]			//WAIT表示の画像
#define IMAGE_TITLE image[9]		//タイトル画像
#define IMAGE_START image[10]		//スタートボタン
#define IMAGE_HELP image[11]		//ヘルプボタン
#define IMAGE_END image[12]			//エンドボタン
#define IMAGE_HELP_B image[13]		//ヘルプ画像
#define IMAGE_BACK_S image[14]		//戻るボタン
#define IMAGE_PRINT image[15]		//インターフェイス用画像
#define IMAGE_CLEAR image[16]		//クリアー時の画像
#define IMAGE_ICON image[17]		//アイコン画像
#define IMAGE_NUMBER image[18]		//数字画像
#define IMAGE_COLOR image[19]		//色変更用画像

#define IMAGE_ENEMY_W 32				//敵の画像幅
#define IMAGE_ENEMY_H 32				//敵の画像高さ
#define IMAGE_AMMO_W 4				//残弾の画像幅
#define IMAGE_AMMO_H 16				//残弾の画像高さ
#define IMAGE_MAX 20					//イメージ保存数
#define ENEMY_MAX 20					//エネミー最大数
#define ENEMY_NONE -999				//敵は存在しない
#define ENEMY_DEAD -935				//敵が死んでいる
#define SCENE_TITLE 0				//シーン＿タイトル画面
#define SCENE_HELP 1					//シーン＿ヘルプ画面
#define SCENE_PLAY 2					//シーン＿プレイ画面
#define SCENE_CLEAR 3				//シーン＿クリア画面
#define ENEMY_ANIMATION_SPEED 5		//アニメーションのスピード
#define ENEMY_ANIMATION_PATTERN 4	//アニメーションパターン
#define ENEMY_ANIMATION_DEAD 20		//死亡時アニメーション
#define ENEMY_HP 6					//敵のHP
#define ENEMY_BLUE 0					//敵＿青色
#define ENEMY_RED 1					//敵＿赤色
#define ENEMY_GREEN 2				//敵＿緑色
#define ENEMY_PURPLE 3				//敵＿紫色
#define RELOAD_TIME 64				//リロードに必要な時間
#define TIMER_START 3660				//タイム

#define DAMAGE_HEAD 6				//頭を撃ったときのダメージ
#define DAMAGE_BODY 4				//体を撃ったときのダメージ
#define DAMAGE_FOOT 2				//足を撃ったときのダメージ
#define HIT_HEAD 3					//頭にヒットした
#define HIT_BODY 2					//体にヒットした
#define HIT_FOOT 1					//足にヒットした

#define EFFECT_MAX 100				//エフェクトの最大数
#define EFFECT_NONE -999				//エフェクトが存在しない
#define HIT_GREEN 4					//エフェクト＿頭にヒット
#define HIT_YELLOW 5					//エフェクト＿体にヒット
#define HIT_RED 6						//エフェクト＿足にヒット
#define COLOR_MAX 6					//カーソル色の個数
#define AMMO_MAX 32					//最大弾数

#define FPS 60						//FPS
#define INTERVAL 1000 / FPS

//敵に対するヒットチェック
#define HITCHECK(a, b, c, d) (mouse_pos.x >= mob[i].pos.x + (a) && mouse_pos.x < mob[i].pos.x + (b) && mouse_pos.y >= mob[i].pos.y + (c) && mouse_pos.y < mob[i].pos.y + (d))
//ボタンに対するヒットチェック
#define BUTTON_CHECK(xpos, ypos, width, height) (mouse_pos.x >= (xpos) && mouse_pos.x < (xpos) + (width) && mouse_pos.y >= (ypos) && mouse_pos.y < (ypos) + (height))

//敵のデータ構造体
struct MOB_DATA {
	SDL_Rect pos; //X座標, Y座標
	int animation; //アニメーション
	int hp; //HP管理＋フラグ
	int id;
	float z; //Z座標
	float y; //表示用のY座標
};

//エフェクトのデータ構造体
struct EFFECT_DATA {
	SDL_Rect pos; 		//X座標, Y座標
	int id; 		//ID 0‾3がヒット時エフェクト 4‾6が緑黄赤 -999で存在しない
	int animation;		//アニメーション(主に透過度)
};


void fps_wait(void);
int HitEffect(struct EFFECT_DATA *effect, int color, SDL_Rect mouse_pos);
int Hit_Proc(struct MOB_DATA *mob, int *score, int *number, int hitplace, int *kill_count);
void GunEffect(struct EFFECT_DATA *effect, SDL_Rect mouse_pos);
void SetImagePos(SDL_Surface image, SDL_Surface screen, Sint16 dx, Sint16 dy, Sint16 sx, Sint16 sy, Uint16 sw, Uint16 sh);
